@@ -631,6 +631,8 @@ sub hostport {
 sub find_apache_module {
     my($self, $module) = @_;
 
+    die "find_apache_module: module name argument is required" unless $module;
+
     my $vars = $self->{vars};
     my $sroot = $vars->{serverroot};
 
@@ -780,13 +782,20 @@ sub perlscript_header {
 
     require FindBin;
 
-    # the live 'lib/' dir of the distro (e.g. modperl-2.0/ModPerl-Registry/lib)
-    my @dirs = canonpath catdir $FindBin::Bin, "lib";
+    my @dirs = ();
 
-    # the live dir of the top dir if any  (e.g. modperl-2.0/lib)
-    if (-e catfile($FindBin::Bin, "..", "Makefile.PL") &&
-        -d catdir($FindBin::Bin, "..", "lib") ) {
-        push @dirs, canonpath catdir $FindBin::Bin, "..", "lib";
+    # mp2 needs its modper-2.0/lib before blib was created
+    if (IS_MOD_PERL_2_BUILD || $ENV{APACHE_TEST_LIVE_DEV}) {
+        # the live 'lib/' dir of the distro
+        # (e.g. modperl-2.0/ModPerl-Registry/lib)
+        my $dir = canonpath catdir $FindBin::Bin, "lib";
+        push @dirs, $dir if -d $dir;
+
+        # the live dir of the top dir if any  (e.g. modperl-2.0/lib)
+        if (-e catfile($FindBin::Bin, "..", "Makefile.PL")) {
+            my $dir = canonpath catdir $FindBin::Bin, "..", "lib";
+            push @dirs, $dir if -d $dir;
+        }
     }
 
     for (qw(. ..)) {
@@ -1259,7 +1268,7 @@ sub generate_httpd_conf {
                 $entry = qq(Include "$file");
             }
             elsif ($file =~ /\.pl$/) {
-                $entry = qq(PerlRequire "$file");
+                $entry = qq(<IfModule mod_perl.c>\n    PerlRequire "$file"\n</IfModule>\n);
             }
             else {
                 next;
@@ -1315,10 +1324,12 @@ sub generate_httpd_conf {
 EOF
     }
 
+    print $out "<IfModule mod_alias.c>\n";
     for (keys %aliases) {
         next unless $vars->{$aliases{$_}};
-        print $out "Alias /getfiles-$_ $vars->{$aliases{$_}}\n";
+        print $out "    Alias /getfiles-$_ $vars->{$aliases{$_}}\n";
     }
+    print $out "</IfModule>\n";
 
     print $out "\n";
 
@@ -1440,8 +1451,18 @@ sub add_inc {
     # make sure that Apache-Test/lib will be first in @INC,
     # followed by modperl-2.0/lib (or some other project's lib/),
     # followed by blib/ and finally system-wide libs.
-    lib::->import(map "$self->{vars}->{top_dir}/$_",
-                  qw(Apache-Test/lib lib blib/lib blib/arch));
+    my $top_dir = $self->{vars}->{top_dir};
+    my @dirs = map { catdir $top_dir, "blib", $_ } qw(lib arch);
+
+    my $apache_test_dir = catdir $top_dir, "Apache-Test";
+    unshift @dirs, $apache_test_dir if -d $apache_test_dir;
+
+    if ($ENV{APACHE_TEST_LIVE_DEV}) {
+        my $lib_dir = catdir $top_dir, "lib";
+        push @dirs, $lib_dir if -d $lib_dir;
+    }
+
+    lib::->import(@dirs);
     #print join "\n", "add_inc", @INC, "";
 }
 
@@ -1652,6 +1673,29 @@ automatically removed on cleanup if found empty.
 
 =back
 
+=head1 Environment Variables
+
+The following environment variables affect the configuration and the
+run-time of the C<Apache::Test> framework:
+
+=head2 APACHE_TEST_COLOR
+
+To aid visual control over the configuration process and the run-time
+phase, C<Apache::Test> uses coloured fonts when the environment
+variable C<APACHE_TEST_COLOR> is set to a true value.
+
+=head2 APACHE_TEST_LIVE_DEV
+
+When using C<Apache::Test> during the project development phase, it's
+often convenient to have the I<project/lib> (live) directory appearing
+first in C<@INC> so any changes to the Perl modules, residing in it,
+immediately affect the server, without a need to rerun C<make> to
+update I<blib/lib>. When the environment variable
+C<APACHE_TEST_LIVE_DEV> is set to a true value during the
+configuration phase (C<t/TEST -config>, C<Apache::Test> will
+automatically unshift the I<project/lib> directory into C<@INC>, via
+the autogenerated I<t/conf/modperl_inc.pl> file.
+
 =head1 AUTHOR
 
 =head1 SEE ALSO
@@ -1711,6 +1755,12 @@ HostnameLookups Off
     MaxRequestsPerChild  0
 </IfModule>
 
+<IfDefine APACHE1>
+    StartServers         1
+    MaxClients           @MaxClients@
+    MaxRequestsPerChild  0
+</IfDefine>
+
 <IfModule mpm_winnt.c>
     ThreadsPerChild      20
     MaxRequestsPerChild  0
