@@ -13,6 +13,7 @@ use Apache::TestTrace;
 
 use File::Find qw(finddepth);
 use File::Spec::Functions qw(catfile);
+use File::Basename qw(basename);
 use Getopt::Long qw(GetOptions);
 use Config;
 
@@ -427,6 +428,10 @@ sub try_exit_opts {
         }
         else {
             warning "server $self->{server}->{name} is not running";
+            # cleanup a stale httpd.pid file if found
+            my $t_logs  = $self->{test_config}->{vars}->{t_logs};
+            my $pid_file = catfile $t_logs, "httpd.pid";
+            unlink $pid_file if -e $pid_file;
         }
         exit_perl $ok;
     }
@@ -568,6 +573,9 @@ sub run {
 
     $self->{test_config} = $self->new_test_config;
 
+    # make it easy to move the whole distro
+    $self->refresh unless -e $self->{test_config}->{vars}->{top_dir};
+
     $self->warn_core();
 
     $self->{server} = $self->{test_config}->server;
@@ -621,14 +629,77 @@ sub oh {
 #e.g. t/core or t/core.12499
 my $core_pat = '^core(\.\d+)?' . "\$";
 
+# $self->scan_core_incremental([$only_top_dir])
+# normally would be called after each test
+# and since it updates the list of seen core files
+# scan_core() won't report these again
+# currently used in Apache::TestSmoke
+#
+# if $only_t_dir arg is true only the t_dir dir (t/) will be scanned
+sub scan_core_incremental {
+    my($self, $only_t_dir) = @_;
+    my $vars = $self->{test_config}->{vars};
+
+    # no core files dropped on win32
+    return () if Apache::TestConfig::WIN32;
+
+    if ($only_t_dir) {
+        require IO::Dir;
+        my @cores = ();
+        for (IO::Dir->new($vars->{t_dir})->read) {
+            next unless -f;
+            next unless /$core_pat/o;
+            my $core = catfile $vars->{t_dir}, $_;
+            next if exists $core_files{$core} && $core_files{$core} == -M $core;
+            $core_files{$core} = -M $core;
+            push @cores, $core;
+        }
+        return @cores 
+            ? join "\n", "server dumped core, for stacktrace, run:",
+                map { "gdb $vars->{httpd} -core $_" } @cores
+            : ();
+    }
+
+    my @msg = ();
+    finddepth({ no_chdir => 1,
+                wanted   => sub {
+        return unless -f $_;
+        my $file = basename $File::Find::name;
+        return unless $file =~ /$core_pat/o;
+        my $core = $File::Find::name;
+        unless (exists $core_files{$core} && $core_files{$core} == -M $core) {
+            # new core file!
+
+            # XXX: could rename the file if it doesn't include the pid
+            # in its name (i.e., just called 'core', instead of 'core.365')
+
+            # XXX: could pass the test name and rename the core file
+            # to use that name as a suffix, plus pid, time or some
+            # other unique identifier, in case the same test is run
+            # more than once and each time it caused a segfault
+            $core_files{$core} = -M $core;
+            push @msg, "server dumped core, for stacktrace, run:\n" .
+                "gdb $vars->{httpd} -core $core";
+        }
+    }}, $vars->{top_dir});
+
+    return @msg;
+
+}
+
 sub scan_core {
     my $self = shift;
     my $vars = $self->{test_config}->{vars};
     my $times = 0;
 
-    finddepth(sub {
+    # no core files dropped on win32
+    return if Apache::TestConfig::WIN32;
+
+    finddepth({ no_chdir => 1,
+                wanted   => sub {
         return unless -f $_;
-        return unless /$core_pat/o;
+        my $file = basename $File::Find::name;
+        return unless $file =~ /$core_pat/o;
         my $core = $File::Find::name;
         if (exists $core_files{$core} && $core_files{$core} == -M $core) {
             # we have seen this core file before the start of the test
@@ -640,7 +711,7 @@ sub scan_core {
             error "oh $oh, server dumped core $again";
             error "for stacktrace, run: gdb $vars->{httpd} -core $core";
         }
-    }, $vars->{top_dir});
+    }}, $vars->{top_dir});
 }
 
 # warn the user that there is a core file before the tests
@@ -651,6 +722,9 @@ sub warn_core {
     my $vars = $self->{test_config}->{vars};
     %core_files = (); # reset global
 
+    # no core files dropped on win32
+    return if Apache::TestConfig::WIN32;
+
     finddepth(sub {
         return unless -f $_;
         return unless /$core_pat/o;
@@ -673,6 +747,11 @@ sub warn_core {
 #    have the right permissions to write into the fs created by 'foo'.
 #
 # We solve that by 'chown -R bar.bar t/' in a portable way.
+#
+# 3. If the parent directory is not rwx for the chosen user, that user
+#    won't be able to read/write the DocumentRoot. In which case we
+#    have nothing else to do, but to tell the user to fix the situation.
+#
 sub adjust_t_perms {
     my $self = shift;
 
@@ -686,11 +765,16 @@ sub adjust_t_perms {
         my $user = $vars->{user};
         my($uid, $gid) = (getpwnam($user))[2..3]
             or die "Can't find out uid/gid of '$user'";
-        warning "root mode: changing the fs ownership to '$user' ($uid:$gid)";
+
+        warning "root mode: ". 
+            "changing the files ownership to '$user' ($uid:$gid)";
         finddepth(sub {
             $original_t_perms{$File::Find::name} = [(stat $_)[4..5]];
             chown $uid, $gid, $_;
         }, $vars->{t_dir});
+
+        $self->check_perms($user, $uid, $gid);
+
     }
 }
 
@@ -700,6 +784,7 @@ sub restore_t_perms {
     return if Apache::TestConfig::WINFU;
 
     if (%original_t_perms) {
+        warning "root mode: restoring the original files ownership";
         my $vars = $self->{test_config}->{vars};
         while (my($file, $ids) = each %original_t_perms) {
             next unless -e $file; # files could be deleted
@@ -708,6 +793,38 @@ sub restore_t_perms {
     }
 }
 
+sub check_perms {
+    my ($self, $user, $uid, $gid) = @_;
+
+    # test that the base dir is rwx by the selected non-root user
+    my $vars = $self->{test_config}->{vars};
+    my $dir  = $vars->{t_dir};
+    my $perl = $vars->{perl};
+    my $check = qq[su -m $user -c '$perl -e ] .
+        qq["print -r q{$dir} &&  -w _ && -x _ ? q{OK} : q{NOK}"'];
+    warning "$check\n";
+    my $res   = qx[$check] || '';
+    warning "result: $res";
+    unless ($res eq 'OK') {
+        #$self->restore_t_perms;
+        error(<<"EOI") && die "\n";
+You are running the test suite under user 'root'.
+Apache cannot spawn child processes as 'root', therefore
+we attempt to run the test suite with user '$user' ($uid:$gid).
+The problem is that the path:
+  $dir
+must be 'rwx' by user '$user', so Apache can read and write under that
+path.
+
+There several ways to resolve this issue. For example move 
+'$dir' to '/tmp/' and repeat the 'make test' phase. 
+
+You can test whether the location is good by running the following test:
+  % $check
+EOI
+    }
+}
+
 sub run_request {
     my($test_config, $opts) = @_;
 
@@ -872,3 +989,50 @@ sub exit_shell {
 }
 
 1;
+
+__END__
+
+=head1 NAME
+
+Apache::TestRun - Run the test suite
+
+=head1 SYNOPSIS
+
+
+=head1 DESCRIPTION
+
+The C<Apache::TestRun> package controls the configuration and running
+of the test suite.
+
+=head1 METHODS
+
+Several methods are sub-classable, if the default behavior should be
+changed.
+
+=head2 C<pre_configure>
+
+The C<pre_configure()> method is executed before the configuration for
+C<Apache::Test> is generated. So if you need to adjust the setup
+before I<httpd.conf> and other files are autogenerated, this is the
+right place to do so.
+
+For example if you don't want to inherit a LoadModule directive for
+I<mod_apreq.so> but to make sure that the local version is used, you
+can sub-class C<Apache::TestRun> and override this method in
+I<t/TEST.PL>:
+
+  package My::TestRun;
+  use base 'Apache::TestRun';
+  use Apache::TestConfig;
+  __PACKAGE__->new->run(@ARGV);
+  
+  sub pre_configure {
+      my $self = shift;
+      # Don't load an installed mod_apreq
+      Apache::TestConfig::autoconfig_skip_module_add('mod_apreq.c');
+  }
+
+Notice that the extension is I<.c>, and not I<.so>.
+
+
+=cut
